@@ -4,11 +4,10 @@ use Mouse;
 
 use warnings;
 use strict;
-use Carp;
 use Storable qw/nstore retrieve/;
 use File::Temp qw/tempdir/;
-use File::Which;
 use File::Spec;
+use Config::Auto;
 use Data::Dumper;
 use Bio::Grid::Run::SGE::Index;
 use Bio::Grid::Run::SGE::Iterator;
@@ -17,21 +16,39 @@ use Cwd qw/fastcwd/;
 use Clone qw/clone/;
 use Data::Printer colored => 1, use_prototypes => 0;
 use Bio::Gonzales::Util::Cerial;
-use Capture::Tiny 'capture';
 use Config;
-use FindBinNew qw($Bin $Script);
-FindBinNew::again();
+use FindBin;
 
-our $VERSION = '0.0387_01'; # VERSION
+our $VERSION = 0.01_01;
+our $RC_FILE = "$ENV{HOME}/.bio-grid-run-sge.conf";
 
-has 'cmd' => (
-  is       => 'rw',
-  required => 1,
-  isa      => 'ArrayRef[Str]',
-  default  => sub {
-    ["$Bin/$Script"];
+override 'BUILDARGS' => sub {
+  my ($self) = @_;
+  my $a = super();
+
+  my $c;
+  $c = eval { Config::Auto::parse($RC_FILE) } if ( $RC_FILE && -f $RC_FILE );
+  if ( $c && !$@ ) {
+    print STDERR "Using config from " . $RC_FILE . "\n";
+    $a = { %{$c}, %{$a} };
   }
-);
+
+  if ( exists( $a->{'config'} ) ) {
+    $c = eval { Config::Auto::parse( $a->{'config'} ) };
+    unless ($@) {
+      print STDERR "Using config from " . $a->{'config'} . "\n";
+      $a = { %{$c}, %{$a} };
+    }
+  }
+  if ( exists( $a->{method} ) && !exists( $a->{mode} ) ) {
+    warn "The configuration option 'method' is DEPRECATED, use 'mode' instead.";
+    $a->{mode}   = $a->{method};
+    $a->{method} = "DEPRECATED: The configuration option 'method' is DEPRECATED, use 'mode' instead.";
+  }
+  return $a;
+};
+
+has 'cmd' => ( is => 'rw', required => 1, isa => 'ArrayRef[Str]' );
 has 'no_post_task' => ( is => 'rw' );
 
 has 'tmp_dir'    => ( is => 'rw', lazy_build => 1 );
@@ -41,10 +58,11 @@ has 'result_dir' => ( is => 'rw', lazy_build => 1 );
 has 'log_dir'    => ( is => 'rw', lazy_build => 1 );
 has 'idx_dir'    => ( is => 'rw', lazy_build => 1 );
 has 'test'       => ( is => 'rw' );
-has 'notify'     => ( is => 'rw' );
-has 'no_prompt'  => ( is => 'rw' );
-has 'lib'        => ( is => 'rw' );
-has 'script_dir' => ( is => 'ro', default    => $Bin );
+has 'mail'       => ( is => 'rw' );
+has 'smtp_server' => ( is => 'rw' );
+has 'no_prompt'   => ( is => 'rw' );
+has 'lib'         => ( is => 'rw' );
+has 'script_dir' => ( is => 'ro', default => $FindBin::Bin);
 
 has 'input' => ( is => 'rw', isa => 'ArrayRef', default => sub { [] } );
 
@@ -65,7 +83,7 @@ has 'submit_bin'          => ( is => 'rw', default    => 'qsub' );
 has 'submit_params'       => ( is => 'rw', default    => sub { [] }, isa => 'ArrayRef[Str]' );
 has 'perl_bin'            => ( is => 'rw', default    => $Config{perlpath} );
 has 'working_dir'         => ( is => 'rw', default    => '.' );
-has 'prefix_output_dirs'  => ( is => 'rw', default    => 1 );
+has 'prefix_output_dirs' => ( is => 'rw' );
 
 # arguments for the cluster script
 has 'args' => ( is => 'rw', isa => 'ArrayRef[Str]', default => sub { [] } );
@@ -131,9 +149,6 @@ sub BUILD {
 
   #confess "No input given" unless ( @{ $self->input } > 0 );
 
-  my $submit_bin = -f $self->submit_bin ? $self->submit_bin : which( $self->submit_bin );
-  confess "[SUBMIT_ERROR] $submit_bin not found or not executable" unless ( -x $submit_bin );
-
   for my $i ( @{ $self->input } ) {
     #merge different namings to one std. naming: elements
     for my $key (qw/list files/) {
@@ -145,16 +160,17 @@ sub BUILD {
   }
 
   $self->perl_bin( expand_path_rel( $self->perl_bin ) );
+  confess "working dir not correct " . $self->working_dir unless ( -d $self->working_dir );
 
-  $self->working_dir( File::Spec->rel2abs( $self->working_dir ) );
-  confess "working dir does not exist: " . $self->working_dir unless ( -d $self->working_dir );
-
-  for my $d (qw/log_dir stderr_dir stdout_dir result_dir tmp_dir idx_dir/) {
+  my $curdir = fastcwd;
+  #chdir $self->working_dir;
+  for my $d (qw/working_dir log_dir stderr_dir stdout_dir result_dir tmp_dir idx_dir/) {
     $self->$d( expand_path( $self->$d ) );
     unless ( -d $self->$d ) {
       my_mkdir( $self->$d );
     }
   }
+  chdir $curdir;
 
   my $m = __PACKAGE__->meta;
 
@@ -167,6 +183,21 @@ sub BUILD {
   $self->extra( { %extra, %{ $self->extra } } );
 }
 
+sub _unknown_attrs_to_extra {
+  my ( $self, $c ) = @_;
+  my $m = __PACKAGE__->meta;
+
+  $c->{extra} //= {};
+  my %attrs = map { $_->name => 1 } $m->get_all_attributes;
+  for my $k ( keys %$c ) {
+    unless ( exists( $attrs{$k} ) ) {
+      $c->{extra}{$k} = delete $c->{$k};
+    }
+  }
+
+  return $c;
+}
+
 sub to_string {
   my ($self) = @_;
 
@@ -203,7 +234,7 @@ sub _build__worker_config_file {
 
 sub _build__worker_env_script {
   my $self = shift;
-  return File::Spec->catfile( $self->tmp_dir, join( '.', 'env', $self->job_name, 'pl' ) );
+  return File::Spec->catfile( $self->tmp_dir, join( '', $self->job_name, '.env.pl' ) );
 }
 
 sub generate_idx_file_name {
@@ -241,33 +272,19 @@ sub run {
 
   say STDERR "Running: " . $cmd;
 
-  # capture from external command
+  my $res = qx/$cmd/;
+  $res =~ /^Your\s*job(-array)?\s*(\d+)/;
+  $self->job_id($2);
 
-  my ( $stdout, $stderr, $exit ) = capture {
-    system(@$cmd_args);
-  };
-
-  if ( $exit != 0 ) {
-    confess "[SUBMIT_ERROR] Could not submit job:\n$stdout$stderr";
-  }
-
-  $stdout =~ /^Your\s*job(-array)?\s*(\d+)/;
-
-  unless ( defined $self->job_id ) {
-    if ( defined $2 ) {
-      $self->job_id($2);
-    } else {
-      warn "[SUBMIT_WARNING] could not parse job id, using -1 as job id.\nSTDOUT:\n$stdout\nSTDERR:\n$stderr";
-      $self->job_id(-1);
-    }
-  }
+  my $tmp_job_id = -1;
+  $tmp_job_id = $2 if defined $2;
 
   open my $main_fh, '>',
-    File::Spec->catfile( $self->log_dir, sprintf( "main.%s.j%s.cmd", $self->job_name, $self->job_id ) )
+    File::Spec->catfile( $self->log_dir, sprintf( "main.%s.j%s.cmd", $self->job_name, $tmp_job_id ) )
     or confess "Can't open filehandle: $!";
   print $main_fh "cd '" . fastcwd . "' && " . $cmd, "\n";
-  $main_fh->close;
-  $self->queue_post_task($config_file) if ( $self->job_id >= 0 );
+  close $main_fh;
+  $self->queue_post_task($config_file) if ( $self->job_id );
 
   return { config => $c, command => $cmd_args };
 }
@@ -344,16 +361,15 @@ EOS
       if ( @inc_dirs && @inc_dirs > 0 );
   }
 
-  print $fh <<'EOF';
-  my $cmd = shift;
-  unless ( my $return = do $cmd ) {
-    warn "could not parse $cmd\n$@\n\n$!" if $@;
-    warn "couldn't execute $cmd\n$!" unless defined $return;
-    warn "couldn't run $cmd" unless $return;
-  }
-  exit;
-EOF
-  $fh->close;
+  print $fh 'my $cmd = shift;',                        "\n";
+  print $fh 'unless ( my $return = do $cmd ) {',       "\n";
+  print $fh '  warn "could not parse $cmd $@" if $@;', "\n";
+  # not necessary
+  print $fh '  warn "could not do $cmd $!" unless defined $return;', "\n";
+  print $fh '  warn "could not run $cmd" unless $return;',           "\n";
+  print $fh '}',                                                     "\n";
+  print $fh 'exit;',                                                 "\n";
+  close $fh;
 
   return $self->_worker_env_script;
 }
@@ -382,7 +398,7 @@ sub queue_post_task {
 
   open my $post_fh, '>', $post_cmd_file or confess "Can't open filehandle: $!";
   print $post_fh join( " ", "cd", "'" . fastcwd . "'", '&&', @cmd, @post_cmd ), "\n";
-  $post_fh->close;
+  close $post_fh;
 
   chmod 0755, $post_cmd_file;
 
@@ -401,7 +417,7 @@ sub save_config {
   open my $cfg_fh, '>', $cfg_save
     or confess "Can't open filehandle: $!";
   print $cfg_fh Dumper($self);
-  $cfg_fh->close;
+  close $cfg_fh;
 
   return;
 }
